/*******************************************************************************
 *   Filename:       bsp_timer.c
 *   Revised:        All copyrights reserved to Roger-WY.
 *   Revision:       v1.0
 *   Writer:	     Roger-WY.
 *
 *   Description:    软定时器模块
 *                   设置systick中断，实现若干个软件定时器。
 *   Notes:
 *     				E-mail:261313062@qq.com
 *
 *******************************************************************************/

#include "bsp_softtimer.h"


/* 这2个全局变量转用于 Bsp_DelayNms() 函数 */
static volatile uint32_t s_uiDelayCount  = 0;
static volatile uint8_t  s_ucTimeOutFlag = 0;

/* 定于软件定时器结构体变量 */
static SOFT_TMR s_tTmr[TMR_COUNT];

/*
	全局运行时间，单位1ms
	最长可以表示 24.85天，如果你的产品连续运行时间超过这个数，则必须考虑溢出问题
*/
volatile int32_t g_iRunTime = 0;

//----------------------------------------------------------------------------//

static void bsp_SoftTimerDec(SOFT_TMR *_tmr);



//============================================================================//

/*******************************************************************************
 * 名    称： bsp_TimerInit
 * 功    能： 初始化滴答定时器，清空所有软定时器
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 配置滴答定时器1ms中断一次
 *******************************************************************************/
void bsp_SoftTimerInit(void)
{
	uint8_t i;

	/* 清零所有的软件定时器 */
	for (i = 0; i < TMR_COUNT; i++) {
		s_tTmr[i].Count     = 0;
		s_tTmr[i].PreLoad   = 0;
		s_tTmr[i].Flag      = 0;
		s_tTmr[i].Mode      = TMR_ONCE_MODE;	/* 缺省是1次性工作模式 */
	}
}


/*******************************************************************************
 * 名    称： bsp_Timer1msCallBack
 * 功    能： 1ms执行一次的中断服务函数
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
extern void bsp_RunPer1ms(void);
extern void bsp_RunPer10ms(void);
void bsp_Timer1msCallBack(void)
{
	static uint8_t s_count = 0;
	uint8_t i;

	/* 每隔1ms进来1次 （仅用于 Bsp_DelayNms） */
	if (s_uiDelayCount > 0) {
		if (--s_uiDelayCount == 0) {
			s_ucTimeOutFlag = 1;
		}
	}

	/* 每隔1ms，对软件定时器的计数器进行减一操作 */
	for (i = 0; i < TMR_COUNT; i++) {
		bsp_SoftTimerDec(&s_tTmr[i]);
	}

	/* 全局运行时间每1ms增1 */
	g_iRunTime++;
	if (g_iRunTime == 0x7FFFFFFF) {	/* 这个变量是 int32_t 类型，最大数为 0x7FFFFFFF */
		g_iRunTime = 0;
	}

	bsp_RunPer1ms();		        /* 每隔1ms调用一次此函数，此函数在 bsp.c */

	if (++s_count >= 10) {
		s_count = 0;
		bsp_RunPer10ms();	        /* 每隔10ms调用一次此函数，此函数在 bsp.c */
	}
}

/*******************************************************************************
 * 名    称： bsp_SoftTimerDec
 * 功    能： 每隔1ms对所有定时器变量减1。
 * 入口参数： _tmr : 定时器变量指针
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 必须被SysTick_ISR周期性调用。
 *******************************************************************************/
static void bsp_SoftTimerDec(SOFT_TMR *_tmr)
{
	if (_tmr->Count > 0) {
		/* 如果定时器变量减到1则设置定时器到达标志 */
		if (--_tmr->Count == 0) {
			_tmr->Flag = 1;

			/* 如果是自动模式，则自动重装计数器 */
			if(_tmr->Mode == TMR_AUTO_MODE) {
				_tmr->Count = _tmr->PreLoad;
			}
		}
	}
}


/*******************************************************************************
 * 名    称： bsp_DelayNmsBySoftTimer
 * 功    能： ms级延迟，延迟精度为正负1ms
 * 入口参数： n : 延迟长度，单位1 ms。 n 应大于2
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
extern void bsp_Idle(void);
void bsp_DelayNmsBySoftTimer(uint32_t n)
{
	/* 避免 n = 1 出现主程序死锁 */
    if (n == 0) {
		return;
	} else if (n == 1) {
		n = 2;
	}

	EA = 0;  			/* 关中断 */

	s_uiDelayCount  = n;
	s_ucTimeOutFlag = 0;

	EA = 1;  				/* 开中断 */

	while (1) {
		bsp_Idle();				/* CPU空闲执行的操作， 见 bsp.c 和 bsp.h 文件 */

		/*
			等待延迟时间到
			注意：编译器认为 s_ucTimeOutFlag = 0，所以可能优化错误，因此 s_ucTimeOutFlag 变量必须申明为 volatile
		*/
		if (s_ucTimeOutFlag == 1) {
			break;
		}
	}
}

/*******************************************************************************
 * 名    称： bsp_StartTimer
 * 功    能： 启动一个定时器，并设置定时周期。
 * 入口参数： _id     : 定时器ID，值域【1,TMR_COUNT-1】。用户必须自行维护定时器ID，以避免定时器ID冲突。
 *						定时器ID = 0 已用于bsp_DelayMS()函数
 *			  _period : 定时周期，单位1ms
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 定时时间到达之后，需要再次启动才能继续工作
 *******************************************************************************/
void bsp_StartTimer(uint8_t _id, uint32_t _period)
{
	if (_id >= TMR_COUNT) {
		/* while(1); 死机 */
		return;
	}
	EA = 0;  		            /* 关中断 */

	s_tTmr[_id].Count   = _period;		/* 实时计数器初值 */
	s_tTmr[_id].PreLoad = _period;		/* 计数器自动重装值，仅自动模式起作用 */
	s_tTmr[_id].Flag    = 0;			/* 定时时间到标志 */
	s_tTmr[_id].Mode    = TMR_ONCE_MODE;/* 1次性工作模式 */

    EA = 1;  		                /* 开中断 */
}

/*******************************************************************************
 * 名    称： bsp_StartAutoTimer
 * 功    能： 启动一个自动定时器，并设置定时周期。
 * 入口参数： _id     : 定时器ID，值域【1,TMR_COUNT-1】。用户必须自行维护定时器ID，以避免定时器ID冲突。
 *						定时器ID = 0 已用于bsp_DelayMS()函数
 *			  _period : 定时周期，单位1ms
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 定时时间到达之后，自动重载初始值。
 *******************************************************************************/
void bsp_StartAutoTimer(uint8_t _id, uint32_t _period)
{
	if (_id >= TMR_COUNT) {
		/* 打印出错的源代码文件名、函数名称 */
		//BSP_Printf("Error: file %s, function %s()\r\n", __FILE__, __FUNCTION__);
		while(1); /* 参数异常，死机等待看门狗复位 */
	}

	EA = 0;  		/* 关中断 */

	s_tTmr[_id].Count   = _period;			/* 实时计数器初值 */
	s_tTmr[_id].PreLoad = _period;		    /* 计数器自动重装值，仅自动模式起作用 */
	s_tTmr[_id].Flag    = 0;				/* 定时时间到标志 */
	s_tTmr[_id].Mode    = TMR_AUTO_MODE;	/* 自动工作模式 */

	EA = 1;  			/* 开中断 */
}


/*******************************************************************************
 * 名    称： bsp_StopTimer
 * 功    能： 停止一个定时器
 * 入口参数： _id     : 定时器ID，值域【1,TMR_COUNT-1】。用户必须自行维护定时器ID，以避免定时器ID冲突。
 *						定时器ID = 0 已用于bsp_DelayMS()函数
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 定时时间到达之后，自动重载初始值。
 *******************************************************************************/
void bsp_StopTimer(uint8_t _id)
{
	if (_id >= TMR_COUNT) {
		/* 打印出错的源代码文件名、函数名称 */
		//BSP_Printf("Error: file %s, function %s()\r\n", __FILE__, __FUNCTION__);
		while(1); /* 参数异常，死机等待看门狗复位 */
	}

	EA = 0;  	/* 关中断 */

	s_tTmr[_id].Count   = 0;				/* 实时计数器初值 */
	s_tTmr[_id].Flag    = 0;				/* 定时时间到标志 */
	s_tTmr[_id].Mode    = TMR_ONCE_MODE;	/* 1次性工作模式  */

	EA = 1;  		/* 开中断 */
}

/*******************************************************************************
 * 名    称： bsp_CheckTimer
 * 功    能： 检测定时器是否超时
 * 入口参数： _id     : 定时器ID，值域【1,TMR_COUNT-1】。用户必须自行维护定时器ID，以避免定时器ID冲突。
 *						定时器ID = 0 已用于bsp_DelayMS()函数
 * 出口参数： 返回 0 表示定时未到， 1表示定时到
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 开启软定时器后使用该函数检查定时时间是否到达
 *******************************************************************************/
uint8_t bsp_CheckTimer(uint8_t _id)
{
	if (_id >= TMR_COUNT) {
		return 0;
	}

	if (s_tTmr[_id].Flag == 1) {
		s_tTmr[_id].Flag = 0;
		return 1;
	} else {
		return 0;
	}
}

/*******************************************************************************
 * 名    称： bsp_GetRunTime
 * 功    能： 获取CPU运行时间，单位1ms
 * 入口参数： 无
 * 出口参数： CPU运行时间，单位1ms
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
int32_t bsp_GetRunTime(void)
{
	int runtime;

	EA = 0;  		/* 关中断 */

	runtime = g_iRunTime;	/* 由于在Systick中断被改写，因此关中断进行保护 */

	EA = 1;  		/* 开中断 */

	return runtime;
}

/*******************************************************************************
 * 名    称： Bsp_CheckRunTime
 * 功    能： 计算当前运行时间和给定时刻之间的差值。处理了计数器循环。
 * 入口参数： _LastTime 上个时刻
 * 出口参数： 当前时间和过去时间的差值，单位1ms
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
int32_t bsp_CheckRunTime(int32_t _LastTime)
{
	int32_t now_time;
	int32_t time_diff;

	EA = 0;  	/* 关中断 */

	now_time = g_iRunTime;	/* 这个变量在Systick中断中被改写，因此需要关中断进行保护 */

	EA = 1;  		/* 开中断 */

	if (now_time >= _LastTime) {
		time_diff = now_time - _LastTime;
	} else {
		time_diff = 0x7FFFFFFF - _LastTime + now_time;
	}

	return time_diff;
}
